@@ -1,9 +1,7 @@
-import json
-import sys
-
 import requests
 
-from cameo_claw import it_distinct, it_streaming_pd, it_group, it_filter, server, it_filter_group
+from cameo_claw import it_distinct, it_group, it_filter, server, it_filter_group, it_streaming_pl, it_download, \
+    it_filter_group_mean_count
 import time
 from cameo_claw.log import log
 
@@ -197,7 +195,7 @@ def get_lst_url_by_date(lst_device_id):
 def test_it_download():
     target_directory = './data/topic_download/'
     int_total = len(get_lst_url_taipei_100())
-    for int_progress, done_url in it_streaming_pd(get_lst_url_taipei_100(), target_directory):
+    for int_progress, done_url in it_download(get_lst_url_taipei_100(), target_directory):
         # print_progress(done_url, int_progress, int_total)
         pass
 
@@ -232,8 +230,7 @@ def test_it_group():
 
 def test_it_filter():
     target_directory = './data/topic_filter/'
-    int_total = len(get_lst_url_taipei_100())
-    lst_distinct_column = ['deviceId', 'localTime', 'sensorId']
+    int_total, lst_distinct_column = total_distinct()
     lst_column_match = [['sensorId', 'pm2_5'], ]
     sort_column = 'localTime'
     for int_progress, done_url in it_filter(
@@ -263,8 +260,7 @@ def test_client():
 
 def test_it_filter_group():
     target_directory = './data/topic_filter_group/'
-    int_total = len(get_lst_url_taipei_100())
-    lst_distinct_column = ['deviceId', 'localTime', 'sensorId']
+    int_total, lst_distinct_column = total_distinct()
     lst_column_match = [['sensorId', 'pm2_5'], ['sensorId', 'voc']]
     sort_column = 'localTime'
     lst_group_by_column = ['deviceId']
@@ -274,6 +270,20 @@ def test_it_filter_group():
         pass
 
 
+def test_it_filter_group_mean_count():
+    target_directory = './data/topic_filter_group_mean_count/'
+    int_total, lst_distinct_column = total_distinct()
+    lst_column_match = [['sensorId', 'pm2_5'], ['sensorId', 'voc']]
+    sort_column = 'localTime'
+    lst_group_by_column = ['deviceId', 'sensorId']
+    time_column = 'localTime'
+    value_column = 'value'
+    for int_progress, done_url in it_filter_group_mean_count(
+            get_lst_url_taipei_100(), target_directory, lst_distinct_column,
+            lst_column_match, sort_column, lst_group_by_column, time_column, value_column):
+        pass
+
+
 if __name__ == '__main__':
     # print('sh/go.sh server')
     # print('sh/go.sh client')
@@ -303,4 +313,8 @@ if __name__ == '__main__':
     test_it_filter_group()
     log(f'test_it_group:{time.time() - t}')
 
+    # t = time.time()
+    # test_it_filter_group_mean_count()
+    # log(f'test_it_group:{time.time() - t}')
+
 # sh/go.sh
